from django.contrib import admin
from webutils.datastore.models import *


class DataStoreAdmin(admin.ModelAdmin):
    list_display = ('id', 'date')
    ordering = ('-date',)
admin.site.register(DataStore, DataStoreAdmin)



import os
import urlparse
from django import template
from django.core.files.storage import get_storage_class
from django.core.files.base import ContentFile
from PIL import Image
from cStringIO import StringIO

register = template.Library()


@register.filter
def thumbnail(file, size='104x104', force_save=False):
    x, y = map(int, size.split('x'))
    try:
        filename = file.path
    except NotImplementedError:
        # Some storages don't support path..
        filename = file.name
    filehead, filetail = os.path.split(filename)
    basename, format = os.path.splitext(filetail)
    miniature = basename + '_' + size + format
    miniature_filename = os.path.join(filehead, miniature)
    filehead, filetail = os.path.split(file.url)
    miniature_url = filehead + '/' + miniature
    storage = get_storage_class()()  # Get storage instance
    if storage.exists(miniature_filename) and not force_save:
        return miniature_url

    image = Image.open(file)
    if image.size[0] < x and image.size[1] < y:
        # New size is bigger than original's size! Don't 
        # create new image.
        miniature_url = file.url
    else:
        stream = StringIO()
        image.thumbnail([x, y], Image.ANTIALIAS)
        image.save(stream, image.format, quality=90)
        storage.save(miniature_filename, ContentFile(stream.getvalue()))

    file.seek(0)  # reset
    return miniature_url



import base64
from webutils.xmlparser import Parser


class Filters(object):
    ''' Base Client Filter Class
        Idea taken from restkit. (http://github.com/benoitc/restkit)
        
        This is a much simpler, and less featureful, system.
    '''
    def __init__(self, filters=[]):
        if filters:
            if isinstance(filters, tuple):
                filters = list(filters)
            if not isinstance(filters, list):
                filters = [filters]
        self.filters = filters or []
        
        # Validate filters
        self.check_filters()
    
    def check_filters(self, filters=None):
        ck_filters = filters or self.filters
        for f in ck_filters:
            if not hasattr(f, 'pre_request') and \
               not hasattr(f, 'post_request'):
                raise TypeError(
                    '%s is not a filter object.' % f.__class__.__name__
                )
    
    def add(self, obj):
        self.check_filters([obj])
        self.filters.append(obj)

    def remove(self, obj):
        for i, f in enumerate(self.filters):
            if obj == f:
                del self.filters[i]

    def apply(self, obj, method='pre_request'):
        for f in self.filters:
            try:
                func = getattr(f, method)
                func(obj)
            except AttributeError:
                continue


class BasicAuthFilter(object):
    ''' Basic Auth Filter '''
    def __init__(self, username, password):
        self.credentials = (username, password)

    def pre_request(self, client):
        encode = base64.b64encode('%s:%s' % self.credentials)
        client.headers.update({'Authorization': 'Basic %s' %  encode})


class XMLParserFilter(object):
    ''' Filter to parse in/out data to/from XML.
    
        Depends on the webutils.xmlparser.Parser class
    '''
    parser = Parser()
    
    def pre_request(self, client):
        if client.request is not None:
            client.request = self.parser.build_request_string(client.request)
    
    def post_request(self, client):
        rdata = client.response.response_data
        client.response.parsed_data = \
                    self.parser.parse(rdata) if rdata else None



from django import template

register = template.Library()


@register.filter
def tabularize(value, cols):
    '''
        modifies a list to become a list of lists 
        eg [1,2,3,4] becomes [[1,2], [3,4]] with an argument of 2
        Taken from django user group

        Usage:
          {% for row in object_list|tabularize:"4" %}
            {% for obj in row %}
              ....
            {% endfor %}
          {% endfor %}
    '''
    try:
        cols = int(cols)
    except ValueError:
        return [value]

    return map(*([None] + [value[i::cols] for i in range(0, cols)]))


class MapTranslateNode(template.Node):
    def __init__(self, data, src_variables, variable_name):
        self.data = data
        self.src_variables = src_variables
        self.variable_name = variable_name

    def render(self, context):
        from webutils.helpers import map_translate
        
        data = template.Variable(self.data).resolve(context)
        src_variables = [
            template.Variable(x).resolve(context) for x in self.src_variables
        ]
        new_data = map_translate(data, *src_variables)

        context[self.variable_name] = new_data
        return u''


@register.tag
def map_translate(parser, token):
    ''' Used to templatize custom data strings. See 
        webutils.helpers.map_translate for more details.
        
        Usage:
        
        {% map_translate data_string, source,extra,fields as result_name %}
        
        Will run data_string through map_translate using source (plus extra
        fields) and give the result in the template context as "result_name"
    '''
    try:
        name, data, src_variables, as_, variable_name = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError(
            'bad arguments for %r' % token.split_contents()[0]
        )

    return MapTranslateNode(data, src_variables.split(','), variable_name)


import cPickle
import datetime
from django.conf import settings
from django.db import models


class DataStore(models.Model):
    dump = models.TextField()
    date = models.DateTimeField(default=datetime.datetime.now)
    updated = models.DateTimeField(default=datetime.datetime.now)

    class Meta:
        ordering = ('-date',)

    def __unicode__(self):
        return u'%i' % (self.id)

    def get_store(self):
        return cPickle.loads(self.dump.encode('ascii'))

    def save(self, *args, **kwargs):
        if self.id:
            self.updated = datetime.datetime.now()
        super(DataStore, self).save(*args, **kwargs)



from django.template import RequestContext
from django.shortcuts import render_to_response

def quick_response(request, template, context={}):
    return render_to_response(
        template,
        context,
        context_instance=RequestContext(request),
    )


'''
    Prints thank you page from an aweber redirect.
    Will give the submitted name, email etc. to the template.

    REQUIRES DJANGO
'''
from django.shortcuts import render_to_response
from django.template import RequestContext


def thankyou(request, template, new_data=None):
    # default config, overwrite with new_data
    data = {
        'name': 'name',
        'email': 'from',
    }

    if new_data is not None and isinstance(new_data, dict):
        data.update(new_data)

    return render_to_response(
        template, {
            'name': request.REQUEST.get(data['name'], 'Name'),
            'email': request.REQUEST.get(data['email'], 'Your Email'),
        },
        context_instance=RequestContext(request)
    )



from django.conf import settings
from django.db import models


class IsActiveManager(models.Manager):
    def __init__(self, field_name='is_active', 
                 field_value=True, *args, **kwargs):
        self.field_name = field_name
        self.field_value = field_value
        super(IsActiveManager, self).__init__(*args, **kwargs)

    def get_query_set(self):
        qs = super(IsActiveManager, self).get_query_set()
        return qs.filter(**{self.field_name: self.field_value})



'''
Used to create maps of template variables in the following format:

{variable}

Into Django readable template variables ({{ variable }})
'''

from django.template import loader, Context
from django.http import HttpResponse


def get_template_source(template_name):
    ''' Return a string and origin for template
    '''
    return loader.find_template_source(template_name)


def load_map(source, vmap={}):
    ''' Create map replacements in given source.
    '''
    for key, value in vmap.items():
        key_str = '{%s}' % key
        val_str = '{{ %s }}' % value
        source = source.replace(key_str, val_str)
    return source


def get_mapped_template(template_name, vmap={}):
    ''' Return a mapped template object
        map should be a dict in the following format:
        
        {'variable': 'django_template_variable',}
        
        This will turn all instances of {variable} in the 
        template source into {{ django_template_variable }}
    '''
    source, origin = get_template_source(template_name)
    source = load_map(source, vmap)
    return loader.get_template_from_string(source, origin, template_name)


def get_mapped_template_from_string(source, vmap={}):
    ''' Return template object from incoming source
    '''
    source = load_map(source, vmap)
    return loader.get_template_from_string(source)


def render_to_mapped_string_from_string(source, dictionary=None, vmap=None):
    ''' Shortcut for string response.
    '''
    dictionary = dictionary or {}
    vmap = vmap or {}
    t = get_mapped_template_from_string(source, vmap)
    return t.render(Context(dictionary))


def render_to_mapped_string(template_name, dictionary=None, 
                                    context_instance=None, vmap=None):
    ''' Shortcut for response usage.
    '''
    dictionary = dictionary or {}
    vmap = vmap or {}
    t = get_mapped_template(template_name, vmap)
    if context_instance:
        context_instance.update(dictionary)
    else:
        context_instance = Context(dictionary)
    return t.render(context_instance)


def render_to_mapped_response(*args, **kwargs):
    ''' Shortcut to create mapped HttpResponse object.
    '''
    httpresponse_kwargs = {'mimetype': kwargs.pop('mimetype', None)}
    return HttpResponse(
        render_to_mapped_string(*args, **kwargs),
        **httpresponse_kwargs
    )


import pyDes
import base64


def handle_crypt(key, data, decrypt=True):
    k = pyDes.triple_des(key)
    if decrypt:
        return k.decrypt(data, '*')
    else:
        return k.encrypt(data, '*')


def encrypt_data(key, data):
    new_data = handle_crypt(key, data, decrypt=False)
    return base64.b64encode(new_data)

def decrypt_data(key, data):
    new_data = base64.b64decode(data)
    return handle_crypt(key, new_data)



import re
import datetime
from django import forms
from django.forms.util import flatatt
from django.utils.encoding import force_unicode
from django.utils.safestring import mark_safe


class TextInputCCEXP(forms.TextInput):
    def render(self, name, value, attrs=None):
        if value is None:
            value = ''
        final_attrs = self.build_attrs(attrs, type=self.input_type, name=name)
        if value != '':
            if isinstance(value, datetime.datetime):
                value = value.date()

            if isinstance(value, datetime.date):
                final_attrs['value'] = force_unicode(value.strftime('%m/%y'))
            else:
                final_attrs['value'] = force_unicode(value)
        return mark_safe(u'<input%s />' % flatatt(final_attrs))


class DateFieldCCEXP(forms.DateField):
    default_error_messages = {
        'invalid': u'Please use the format: MM/YY',
    }
    widget = TextInputCCEXP

    def clean(self, value):
        if value in (None, ''):
            raise forms.ValidationError(self.error_messages['required'])
        if isinstance(value, datetime.datetime):
            return value.date()
        if isinstance(value, datetime.date):
            return value

        if re.search('^\d{1,2}/\d{2}$', value):
            month, year = map(int, value.split('/'))
            return datetime.date((2000 + year), month, 1)

        raise forms.ValidationError(self.error_messages['invalid'])


from django import forms
from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.auth.forms import AuthenticationForm
from django.utils.translation import ugettext_lazy as _


class ResetForm(forms.Form):
    email = forms.EmailField(
        max_length=125,
        label=_('Email Address'),
    )

    def clean_email(self):
        if 'email' in self.cleaned_data:
            email = self.cleaned_data['email']
            if not User.objects.filter(email__iexact=email).count() > 0:
                raise forms.ValidationError(
                    u'There is no account registered to this email address.'
                )
            return email.lower()

        raise forms.ValidationError(u'This field is required.')


class PasswordChangeForm(forms.Form):
    old_password = forms.CharField(
        label=_('Old Password'),
        widget=forms.PasswordInput,
    )
    new_password = forms.CharField(
        label=_('New Password'),
        widget=forms.PasswordInput,
    )
    new_password2 = forms.CharField(
        label=_('Confirm New Password'),
        widget=forms.PasswordInput,
    )
    
    def __init__(self, user, *args, **kwargs):
        self.user = user
        super(PasswordChangeForm, self).__init__(*args, **kwargs)
    
    def clean_old_password(self):
        if 'old_password' in self.cleaned_data:
            oldpw = self.cleaned_data['old_password']
            if not self.user.check_password(oldpw):
                raise forms.ValidationError(_(
                    'Your old password was entered incorrectly. ' + \
                    'Please enter it again.'
                ))
            return oldpw
        raise forms.ValidationError(_('This field is required.'))
    
    def clean_new_password2(self):
        if 'new_password2' in self.cleaned_data:
            pw1 = self.cleaned_data.get('new_password')
            pw2 = self.cleaned_data.get('new_password2')
            if pw1 != pw2:
                raise forms.ValidationError(_(
                    'The two password fields didn\'t match.'
                ))
            return pw2
        raise forms.ValidationError(_('This field is required.'))
    
    def save(self):
        self.user.set_password(self.cleaned_data['new_password'])
        self.user.save()


'''
    Requires django!
    Provide cache functions for django views, etc.
'''
from django.conf import settings
from django.core.cache import cache
from django.http import HttpResponse, HttpRequest
from django.template import Context, loader
from django.core.urlresolvers import reverse, NoReverseMatch
from django.utils.cache import get_cache_key
from django.core.cache.backends.memcached import CacheClass as DjCacheClass


def basic_cached_response(template_file, ctext=None, *args, **kwargs):
    cache_name = '%s_%s' % (settings.ROOT_URLCONF, template_file)
    ctext = ctext or {}
    tmpl = cache.get(cache_name)
    if tmpl is None:
        tmpl = loader.get_template(template_file)
        cache.set(cache_name, tmpl)

    mimetype = {'mimetype': kwargs.pop('mimetype', None)}
    context = kwargs.pop('context_instance', None)
    if context is not None:
        context.update(ctext)
    else:
        context = Context(ctext)

    return HttpResponse(tmpl.render(context), **mimetype)


def expire_view_cache(view_name, args=[], namespace=None, key_prefix=None):
    """
    This function allows you to invalidate any view-level cache. 
        view_name: view function you wish to invalidate or it's 
                   named url pattern
        args: any arguments passed to the view function
        namepace: optioal, if an application namespace is needed
        key prefix: for the @cache_page decorator for the function (if any)
        
        http://stackoverflow.com/questions/2268417/expire-a-view-cache-in-django
    """
    # create a fake request object
    request = HttpRequest()
    
    # Loookup the request path:
    if namespace:
        view_name = namespace + ":" + view_name

    try:
        request.path = reverse(view_name, args=args)
    except NoReverseMatch:
        return False

    # get cache key, expire if the cached item exists:
    key = get_cache_key(request, key_prefix=key_prefix)
    if key:
        if cache.get(key):
            cache.set(key, None, 0)
        return True
    return False


class CacheClass(object):
    ''' Cache class that uses the Django memcached CacheClass
        but adds support for key prefix's. Should be done in 
        the django level, but this is a work around for now.
    '''
    def __init__(self, server, params):
        self._cache = DjCacheClass(server, params)
        self.prefix = getattr(settings, 'CACHE_PREFIX', '')

    def add_prefix(self, key):
        if self.prefix:
            return '%s-%s' % (self.prefix, key)
        return key

    def add(self, key, value, timeout=0):
        key = self.add_prefix(key)
        return self._cache.add(key, value, timeout)

    def get(self, key, default=None):
        key = self.add_prefix(key)
        return self._cache.get(key, default)

    def set(self, key, value, timeout=0):
        key = self.add_prefix(key)
        return self._cache.set(key, value, timeout)

    def delete(self, key):
        key = self.add_prefix(key)
        return self._cache.delete(key)

    def get_many(self, keys):
        keys = map(self.add_prefix, keys)
        return self._cache.get_many(keys)

    def close(self, **kwargs):
        self._cache.close(**kwargs)

    def incr(self, key, delta=1):
        key = self.add_prefix(key)
        return self._cache.incr(key, delta)

    def decr(self, key, delta=1):
        key = self.add_prefix(key)
        return self._cache.decr(key, delta)



from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.auth.decorators import login_required
from django.template import loader, RequestContext
from django.shortcuts import render_to_response, redirect
from webutils.baseacct import Config


def reset(request, reset_form, profile_model, template, key=None):
    if request.user.is_authenticated():
        return redirect(settings.LOGIN_REDIRECT_URL)

    msg = None
    if request.method == 'POST':
        form = reset_form(request.POST)
        if form.is_valid():
            # Form class validates email address is registered 
            # to an account.
            try:
                profile = profile_model.objects.get(
                    user__email__exact=form.cleaned_data['email']
                )
            except profile_model.MultipleObjectsReturned:
                # Shouldn't be reached. Added as sanity check. Grab the 
                # first profile that matches this email address.
                profile = profile_model.objects.filter(
                    user__email__exact=form.cleaned_data['email']
                )[0]

            profile.activation_key = profile_model.objects.generate_hash()
            profile.save()

            subject = 'Confirm your password reset request'
            ctext = {
                'key': profile.activation_key,
            }
            profile_model.objects.send_user_email(
                profile.user, 
                subject, 
                ctext, 
                'baseacct/reset_email.txt',
            )

            msg = ('An email has been sent to %s. Please follow the '
                   'instructions in that email to complete your '
                   'password reset.' % profile.user.email)
            return render_to_response(template, {
                'email': profile.user.email,
                'msg': msg,
                'form': form,
                }, context_instance=RequestContext(request))
    else:
        if key is not None:
            user = profile_model.objects.reset_user_password(key)
            if user:
                msg = ('We have reset your password. You will receive an '
                       'email shortly containing your new password. Use '
                       'that to login to your account.')
            else:
                msg = ('Invalid key was given.')
            return render_to_response(template, {
                'msg': msg,
                'form': reset_form(),
                }, context_instance=RequestContext(request))
        else:
            form = reset_form()

    return render_to_response(template, {
        'form': form,
    }, context_instance=RequestContext(request))


@login_required
def password_change(request, template,
                    password_change_redirect, password_change_form):
    if password_change_redirect is None:
        password_change_redirect = 'baseacct-password-change-done'

    if request.method == 'POST':
        form = password_change_form(request.user, request.POST)
        if form.is_valid():
            form.save()
            return redirect(password_change_redirect)
    else:
        form = password_change_form(request.user)
    
    return render_to_response(template, {
        'form': form,
    }, context_instance=RequestContext(request))


''' Module to easily create zip files on the fly
'''

import zipfile
try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO


class ZipError(Exception):
    ''' General zip error calss'''
    pass


class ZipCreate(object):
    ''' Class to manage the creation of zip files
    '''
    def __init__(self):
        self.is_closed = False
        self.zipdata = StringIO()
        self.zfile = zipfile.ZipFile(self.zipdata, 'w')
    
    def add_from_string(self, filename, data):
        assert self.is_closed is False
        try:
            data = data.encode('utf-8')
        except AttributeError:
            pass
        self.zfile.writestr(filename, data)
    
    def mass_import(self, import_list, is_string=True):
        ''' Takes a list or tuple of 2 list/tuples containing 
            the filename and filedata. IE:
            
            (('file1.txt', 'Data for file'), ('file2.txt', 'More data'))
        '''
        if not isinstance(import_list, (list, tuple)):
            raise ZipError('Data passed was not a list or tuple.')
        
        for item in import_list:
            if is_string:
                self.add_from_string(item[0], item[1])
    
    def close(self):
        self.zfile.close()
        self.is_closed = True
    
    def get_zip_data(self):
        if not self.is_closed:
            self.close()
        return self.zipdata.getvalue()



import gc
from django.conf import settings
from django.template.defaultfilters import slugify
from webutils.helpers import grab_from_import


def import_setting_name(setting_name, default_obj=None):
    ''' Used to import an item from a value in the current 
        Django projects settings.py
        
        Example in settings.py:
        
            FORM_TO_IMPORT = 'myproject.myapp.forms.CustomForm'
        
        Then to grab this, just call import_setting_name:
        
            form = import_setting_name('FORM_TO_IMPORT')
        
        Pass 'default_obj' 
    '''
    if hasattr(settings, setting_name):
        try:
            return grab_from_import(
                getattr(settings, setting_name),
                as_from=True,
            )
        except ImportError, err:
            if default_obj is None:
                raise ImportError(str(err))
    return default_obj


def slugify_uniquely(s, queryset=None, field='slug'):
    '''
    --> Taken from django-crm models.py

    Returns a slug based on 's' that is unique for all instances of the given
    field in the given queryset.
    
    If no string is given or the given string contains no slugify-able
    characters, default to the given field name + N where N is the number of
    default slugs already in the database.
    '''
    new_slug = new_slug_base = slugify(s)
    if queryset is not None:
        queryset = queryset.filter(**{'%s__startswith' % field: new_slug_base})
        similar_slugs = [value[0] for value in queryset.values_list(field)]
        i = 1
        while new_slug in similar_slugs:
            new_slug = "%s%d" % (new_slug_base, i)
            i += 1
    return new_slug


# Taken from http://djangosnippets.org/snippets/1949/
def queryset_iterator(queryset, chunksize=1000):
    ''' Iterate over a Django Queryset ordered by the primary key

        This method loads a maximum of chunksize (default: 1000) rows in it's
        memory at the same time while django normally would load all rows in it's
        memory. Using the iterator() method only causes it to not preload all the
        classes.

        Note that the implementation of the iterator does not support ordered query sets.
        
        Example Usage:

        my_queryset = queryset_iterator(MyModel.objects.all())
        for row in my_queryset:
            print row.pk
    '''
    pk = 0
    last_pk = queryset.order_by('-pk')[0].pk
    queryset = queryset.order_by('pk')
    while pk < last_pk:
        for row in queryset.filter(pk__gt=pk)[:chunksize]:
            pk = row.pk
            yield row
        gc.collect()



'''
    *** DO NOT USE ***
    This was the original "APIClient" I hacked together 
    quickly one day for quick use and testing for an 
    internal project. Over time various apps became 
    dependent on it, which is the only reason it's still 
    here. Yes, I'm embarrassed.
    
    Once I update those apps (one day) to use the new 
    BaseClient + Filters I will remove this from the 
    code repo.
    
    *** DO NOT USE ***
'''
import urllib
import urllib2
import hashlib
import urlparse
from webutils.helpers import encode_dict


class APIError(Exception):
    ''' General exception
    '''
    pass


class APIClient(object):
    def __init__(self, url, key, debug=False):
        url_scheme = urlparse.urlparse(url)
        self.url = url
        self.key = key
        self.scheme = url_scheme.scheme
        self.host = url_scheme.netloc
        self.port = url_scheme.port or 80
        self.path = url_scheme.path
        self.debug = debug

    def create_hash(self, command):
        return hashlib.sha1(self.key + command).hexdigest()

    def send_data(self, params, use_hash=False):
        assert isinstance(params, dict)
        if use_hash and 'command' in params:
            params.update({
                'hash': self.create_hash(params['command']),
            })
        
        return self.send_request(params)

    def send_request(self, request):
        params = urllib.urlencode(encode_dict(request))
        if self.debug:
            print 'DEBUG: ', params

        try:
            uop = urllib2.urlopen(self.url, params)
            code = uop.getcode()
            data = uop.read()
            uop.close()
        except urllib2.HTTPError, err:
            raise APIError(err)

        if code != 200:
            hash = {'statusmsg': '%i - %s' % (code, uop.msg)}
            return False, hash

        if data[:2] != 'OK':
            hash = {
                'statusmsg': data,
            }
            return False, hash

        return True, data


import re
import httplib
import urllib


def gen_form_variables(data):
    '''
        data should be the actual form html..
        This function will pull all the hidden variables 
        and assign them to a dictionary. It can be used 
        in the add_aweber_signup method.
    '''
    data_dict = {}
    pat = r'^<input\s+type="hidden"\s+name="(?P<name>[-\w]+)"' + \
          r'\s+value="(?P<value>.*?)".*?>$'
    for x in data.split('\n'):
        x = x.strip()
        m = re.match(pat, x)
        if m is not None:
            data_dict[m.group('name')] = m.group('value')
    return data_dict


def add_aweber_signup(name, email, url_vars, debug=False):
    '''
        Takes 3 variables...
        name = First name (string)
        email = Email address (string)
        url_vars = aweber form variables (dict)
            url_vars = {
                'meta_web_form_id': '111733762',
                'meta_split_id': '',
                'unit': 'xitplan',
                'redirect': 'http://www.xitplan.com/thankyou.html',
                'meta_redirect_onlist': '',
                'meta_adtracking': 'main index',
                'meta_message': '1',
                'meta_required': 'from',
                'meta_forward_vars': '0',
                'name': name,
                'from': email,
            }
    '''
    url_vars.update({'name': name, 'from': email})

    headers = {
        'Content-type': 'application/x-www-form-urlencoded',
        'Accept': 'text/plain',
    }

    params = urllib.urlencode(url_vars)
    conn = httplib.HTTPConnection('www.aweber.com:80')
    conn.request('POST', '/scripts/addlead.pl', params, headers)
    response = conn.getresponse()
    if debug:
        print response.status, response.reason
    conn.close()



'''
    ClickBank Views
    
    REQUIRES DJANGO
'''
from django.conf import settings
from django.http import HttpResponse
from django.shortcuts import render_to_response
from django.template import RequestContext
from webutils.clickbank.hashproc import thankyou_check


def index(request, template, cb_template):
    '''
        If refer is from Clickbank, print given template.
        If it's not from CB, print other given template.

        Checks for 'hop' in request.GET.
    '''
    affiliate = None
    if 'hop' in request.GET:
        affiliate = request.GET['hop']
    elif 'cb_affiliate' in request.COOKIES:
        affiliate = request.COOKIES['cb_affiliate']

    response = render_to_response(cb_template if affiliate else template, {
                                  'affiliate': affiliate,
                                  }, context_instance=RequestContext(request))

    if affiliate is not None and 'cb_affiliate' not in request.COOKIES:
        # set cookie so if the user returns, the affiliate
        # gets credit.
        response.set_cookie(
            'cb_affiliate',
            value=affiliate,
            max_age=(((60 * 60) * 24) * 62)  # 62 days
        )
    return response


def thankyou(request, template):
    '''
        Used to verify the authenticity of the CB thank you page 
        request.
        *REQUIRES* CB_KEY be set to your clickbank secret key 
        in settings...
    '''
    if not thankyou_check(settings.CB_KEY, request.GET):
        return HttpResponse('ACCESS DENIED')

    return render_to_response(template, {
            'cname': request.GET['cname'].split(' ')[0],
            'cemail': request.GET['cemail'],
        }, context_instance=RequestContext(request))



"""
Class for accepting PayPal's Instant Payment Notification messages in a 
Django application (or Django-on-App-Engine):

Original snippet found here:

http://www.djangosnippets.org/snippets/969/

I've altered it to fit my own personal needs.
"""
import urllib
from django.http import HttpResponse
from django.utils.encoding import smart_str


class Endpoint(object):
    
    default_response_text = 'Nothing to see here'
    verify_url = "https://www.paypal.com/cgi-bin/webscr"
    
    def do_post(self, url, data):
        return urllib.urlopen(url, data).read()

    def verify(self, request):
        raw_post = getattr(request, 'body', request.raw_post_data)
        raw_post += '&cmd=_notify-validate'
        return self.do_post(self.verify_url, raw_post) == 'VERIFIED'
    
    def default_response(self):
        return HttpResponse(self.default_response_text)
    
    def __call__(self, request):
        r = None
        if request.method == 'POST':
            data = dict(request.POST.items())
            # We need to post that BACK to PayPal to confirm it
            if self.verify(request):
                r = self.process(data)
            else:
                r = self.process_invalid(data)
        if r:
            return r
        else:
            return self.default_response()
    
    def process(self, data):
        pass
    
    def process_invalid(self, data):
        pass



from django.conf import settings
from django.contrib.auth.models import User


class EmailBackend(object):
    supports_anonymous_user = False
    supports_inactive_user = False
    supports_object_permissions = False

    def authenticate(self, username=None, password=None):
        if '@' in username:
            kwargs = {'email__iexact': username}
        else:
            kwargs = {'username__iexact': username}

        try:
            user = User.objects.get(**kwargs)
            if user.check_password(password):
                return user
        except User.DoesNotExist:
            return None

    def get_user(self, user_id):
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None



from webutils.baseacct import Config
from django.contrib.auth import views as auth_views
from django.views.generic.simple import direct_to_template
from django.conf.urls.defaults import *


config = Config()

urlpatterns = patterns('webutils.baseacct.views',
    url(r'^login/$',
        auth_views.login, {
            'template_name': 'baseacct/login.html',
            'authentication_form': config.get_login_form(),
        }, name='baseacct-login'),
    url(r'^logout/$',
        auth_views.logout, {
            'template_name': 'baseacct/logout.html',
        }, name='baseacct-logout'),
    url(r'^logout-login/$',
        auth_views.logout_then_login,
        {'login_url': config.get_login_url()},
        name='baseacct-logout-login'),
    url(r'^password_change/$',
        'password_change', {
            'template': 'baseacct/password_change.html',
            'password_change_redirect': config.get_password_change_redirect(),
            'password_change_form': config.get_password_change_form(),
        }, name='baseacct-password-change'),
    url(r'^password_change/done/$',
        direct_to_template, {
            'template': 'baseacct/password_change_done.html',
        }, name='baseacct-password-change-done'),
    url(r'^reset/$',
        'reset', {
            'template': 'baseacct/reset.html',
            'reset_form': config.get_reset_form(),
            'profile_model': config.get_profile_model(),
        }, name='baseacct-reset'),
    url(r'^reset/(?P<key>\w+)/$',
        'reset', {
            'template': 'baseacct/reset_complete.html',
            'reset_form': config.get_reset_form(),
            'profile_model': config.get_profile_model(),
        }, name='baseacct-reset-key'),
)



import urlparse
from django.conf import settings
from django.http import HttpResponseRedirect


def ssl_required(view_func):
    def _checkssl(request, *args, **kwargs):
        if not settings.DEBUG and not request.is_secure():
            if 'HTTP_X_FORWARDED_PROTO' not in request.META:
                # This checks for X_FORWARDED_PROTO header. Usually 
                # passed when SSL is being proxied upstream.
                # This should avoid a redirect loop.
                if hasattr(settings, 'SSL_DOMAIN'):
                    url_str = urlparse.urljoin(
                        settings.SSL_DOMAIN,
                        request.get_full_path()
                    )
                else:
                    url_str = request.build_absolute_uri()
                url_str = url_str.replace('http://', 'https://')
                return HttpResponseRedirect(url_str)

        return view_func(request, *args, **kwargs)
    return _checkssl



''' Cleanup Callbacks
'''
class CleanupCallback(object):
    ''' Base callback class
        Callback can take any data you send it. 
        It *has* to have a handle_file() method that 
        does the work for each file as it's processed.
    '''
    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)
    
    def handle_file(self, path, fname):
        pass


class AWSCallback(CleanupCallback):
    ''' AWS S3 Callback
    '''
    def __init__(self, *args, **kwargs):
        from webutils.aws.s3proc import S3Proc
        super(AWSCallback, self).__init__(*args, **kwargs)

        self.conn = S3Proc(self.aws_key, self.aws_secret_key, self.bucket)
        self.conn.connect()

    def handle_file(self, path, fname):
        pass